from mock import patch
from pytest import raises
import sys
import mock
import kiwi

from ..test_helper import argv_kiwi_tests

from kiwi.defaults import Defaults
from kiwi.builder.archive import ArchiveBuilder

from kiwi.exceptions import KiwiArchiveSetupError


class TestArchiveBuilder:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        self.xml_state = mock.Mock()
        self.xml_state.profiles = None
        self.xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        self.xml_state.get_build_type_name = mock.Mock(
            return_value='tbz'
        )
        self.xml_state.xml_data.get_name = mock.Mock(
            return_value='myimage'
        )
        self.setup = mock.Mock()
        kiwi.builder.archive.SystemSetup = mock.Mock(
            return_value=self.setup
        )
        self.archive = ArchiveBuilder(
            self.xml_state, 'target_dir', 'root_dir'
        )

    def test_create_unknown_archive_type(self):
        xml_state = mock.Mock()
        xml_state.get_build_type_name = mock.Mock(
            return_value='bogus'
        )
        xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        xml_state.xml_data.get_name = mock.Mock(
            return_value='myimage'
        )
        archive = ArchiveBuilder(
            xml_state, 'target_dir', 'root_dir'
        )
        with raises(KiwiArchiveSetupError):
            archive.create()

    @patch('kiwi.builder.archive.ArchiveTar')
    def test_create(self, mock_tar):
        Defaults.set_platform_name('x86_64')
        archive = mock.Mock()
        mock_tar.return_value = archive
        self.archive.create()
        mock_tar.assert_called_once_with(
            'target_dir/myimage.x86_64-1.2.3.tar'
        )
        archive.create_xz_compressed.assert_called_once_with(
            'root_dir', exclude=[
                'image', '.profile', '.kconfig', 'run/*', 'tmp/*',
                '.buildenv', 'var/cache/kiwi'
            ], xz_options=None
        )
        self.setup.export_package_verification.assert_called_once_with(
            'target_dir'
        )
        self.setup.export_package_list.assert_called_once_with(
            'target_dir'
        )

    def teardown(self):
        sys.argv = argv_kiwi_tests



# Copyright (c) 2020 SUSE LLC.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.markup.base import MarkupBase

from kiwi.exceptions import KiwiMarkupConversionError


class MarkupXML(MarkupBase):
    """
    **Implements plain XML markup handling**
    """
    def get_xml_description(self) -> str:
        """
        Return XML description file name

        :return: file path name

        :rtype: string
        """
        return self.apply_xslt_stylesheets(
            self.description
        )

    def get_yaml_description(self) -> str:
        """
        Conversion into YAML format not supported by base XML markup
        """
        raise KiwiMarkupConversionError(
            'Conversion to YAML not supported, install anymarkup module'
        )



from pytest import raises
import mock

from kiwi.firmware import FirmWare
from kiwi.defaults import Defaults

from kiwi.exceptions import KiwiNotImplementedError


class TestFirmWare:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        xml_state = mock.Mock()
        xml_state.build_type.get_firmware = mock.Mock()
        xml_state.build_type.get_firmware.return_value = 'bios'
        self.firmware_bios = FirmWare(xml_state)

        xml_state.build_type.get_efiparttable.return_value = None
        xml_state.build_type.get_efipartsize.return_value = None
        xml_state.build_type.get_firmware.return_value = 'efi'
        self.firmware_efi = FirmWare(xml_state)

        xml_state.build_type.get_efiparttable.return_value = 'msdos'
        xml_state.build_type.get_efipartsize.return_value = None
        xml_state.build_type.get_firmware.return_value = 'efi'
        self.firmware_efi_mbr = FirmWare(xml_state)

        xml_state.build_type.get_efipartsize.return_value = 42
        self.firmware_efi_custom_efi_part = FirmWare(xml_state)

        xml_state.build_type.get_firmware.return_value = 'ec2'
        self.firmware_ec2 = FirmWare(xml_state)

        Defaults.set_platform_name('s390x')
        xml_state.build_type.get_firmware.return_value = None
        xml_state.get_build_type_bootloader_targettype = mock.Mock()

        xml_state.get_build_type_bootloader_targettype.return_value = 'CDL'
        self.firmware_s390_cdl = FirmWare(xml_state)

        xml_state.get_build_type_bootloader_targettype.return_value = 'SCSI'
        self.firmware_s390_scsi = FirmWare(xml_state)

        Defaults.set_platform_name('ppc64le')
        xml_state.build_type.get_firmware.return_value = 'ofw'
        self.firmware_ofw = FirmWare(xml_state)

        xml_state.build_type.get_firmware.return_value = 'opal'
        self.firmware_opal = FirmWare(xml_state)

        Defaults.set_platform_name('x86_64')

    def test_firmware_unsupported(self):
        xml_state = mock.Mock()
        xml_state.build_type.get_firmware = mock.Mock(
            return_value='bogus'
        )
        with raises(KiwiNotImplementedError):
            FirmWare(xml_state)

    def test_get_partition_table_type(self):
        assert self.firmware_bios.get_partition_table_type() == 'msdos'
        assert self.firmware_efi.get_partition_table_type() == 'gpt'
        assert self.firmware_efi_mbr.get_partition_table_type() == 'msdos'
        assert self.firmware_s390_cdl.get_partition_table_type() == 'dasd'
        assert self.firmware_s390_scsi.get_partition_table_type() == 'msdos'

    def test_get_partition_table_type_ppc_ofw_mode(self):
        assert self.firmware_ofw.get_partition_table_type() == 'gpt'

    def test_get_partition_table_type_ppc_opal_mode(self):
        assert self.firmware_opal.get_partition_table_type() == 'gpt'

    def test_legacy_bios_mode(self):
        assert self.firmware_bios.legacy_bios_mode() is False
        assert self.firmware_efi.legacy_bios_mode() is True

    def test_legacy_bios_mode_non_x86_platform(self):
        self.firmware_efi.arch = 'arm64'
        assert self.firmware_efi.legacy_bios_mode() is False

    def test_ec2_mode(self):
        assert self.firmware_ec2.ec2_mode() == 'ec2'
        assert self.firmware_bios.ec2_mode() is None

    def test_efi_mode(self):
        assert self.firmware_bios.efi_mode() is None
        assert self.firmware_efi.efi_mode() == 'efi'

    def test_bios_mode(self):
        assert self.firmware_bios.bios_mode() is True
        assert self.firmware_efi.bios_mode() is False

    def test_ofw_mode(self):
        assert self.firmware_ofw.ofw_mode() is True
        assert self.firmware_bios.ofw_mode() is False

    def test_opal_mode(self):
        assert self.firmware_opal.opal_mode() is True
        assert self.firmware_bios.opal_mode() is False

    def test_get_legacy_bios_partition_size(self):
        assert self.firmware_bios.get_legacy_bios_partition_size() == 0
        assert self.firmware_efi.get_legacy_bios_partition_size() == 2

    def test_get_efi_partition_size(self):
        assert self.firmware_bios.get_efi_partition_size() == 0
        assert self.firmware_efi.get_efi_partition_size() == 20
        assert self.firmware_efi_custom_efi_part.get_efi_partition_size() == 42

    def test_get_prep_partition_size(self):
        assert self.firmware_ofw.get_prep_partition_size() == 8



import logging
from mock import patch
from pytest import (
    raises, fixture
)

from kiwi.utils.codec import Codec

from kiwi.exceptions import KiwiDecodingError


class TestCodec:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    def setup(self):
        self.literal = bytes(b'\xc3\xbc')

    @patch('kiwi.utils.codec.Codec._wrapped_decode')
    def test_decode_ascii_failure(self, mock_decode):
        msg = 'utf-8 compatible string'

        def mocked_decode(literal, encoding, error_handling_schema):
            if encoding:
                return msg
            else:
                raise KiwiDecodingError('ascii decoding failure')

        mock_decode.side_effect = mocked_decode
        with self._caplog.at_level(logging.WARNING):
            assert msg == Codec.decode(self.literal)

    def test_decode_None_literal(self):
        assert '' == Codec.decode(None)

    @patch('kiwi.utils.codec.Codec._wrapped_decode')
    def test_decode(self, mock_decode):
        msg = 'utf-8 compatible string'
        mock_decode.return_value = msg
        assert msg == Codec.decode(self.literal)

    @patch('kiwi.utils.codec.Codec._wrapped_decode')
    def test_decode_utf8_failure(self, mock_decode):
        def mocked_decode(literal, encoding, error_handling_schema):
            if encoding:
                raise KiwiDecodingError('utf-8 decoding failure')
            else:
                raise KiwiDecodingError('ascii decoding failure')

        mock_decode.side_effect = mocked_decode
        with raises(KiwiDecodingError):
            with self._caplog.at_level(logging.WARNING):
                Codec.decode(self.literal)

    def test_wrapped_decode(self):
        assert self.literal.decode() == Codec._wrapped_decode(
            self.literal, 'utf-8'
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import time
import logging
from textwrap import dedent
from typing import List

# project
from kiwi.path import Path
from kiwi.utils.temporary import Temporary
from kiwi.command import Command
from kiwi.exceptions import KiwiUmountBusyError

log = logging.getLogger('kiwi')


class MountManager:
    """
    **Implements methods for mounting, umounting and mount checking**

    If a MountManager instance is used to mount a device the caller
    must care for the time when umount needs to be called. The class
    does not automatically release the mounted device, which is
    intentional

    * :param string device: device node name
    * :param string mountpoint: mountpoint directory name
    """
    def __init__(self, device: str, mountpoint: str = ''):
        self.device = device
        if not mountpoint:
            self.mountpoint_tempdir = Temporary(
                prefix='kiwi_mount_manager.'
            ).new_dir()
            self.mountpoint = self.mountpoint_tempdir.name
        else:
            self.mountpoint = mountpoint

    def bind_mount(self) -> None:
        """
        Bind mount the device to the mountpoint
        """
        if not self.is_mounted():
            Command.run(
                ['mount', '-n', '--bind', self.device, self.mountpoint]
            )

    def mount(self, options: List[str] = []) -> None:
        """
        Standard mount the device to the mountpoint

        :param list options: mount options
        """
        if not self.is_mounted():
            option_list = []
            if options:
                option_list = ['-o'] + options
            Command.run(
                ['mount'] + option_list + [self.device, self.mountpoint]
            )

    def umount_lazy(self) -> None:
        """
        Umount by the mountpoint directory in lazy mode

        Release the mount in any case, however the time when the mounted
        resource is released by the kernel depends on when the resource
        enters the non busy state
        """
        if self.is_mounted():
            Command.run(['umount', '-l', self.mountpoint])

    def umount(self, raise_on_busy: bool = True) -> bool:
        """
        Umount by the mountpoint directory

        Wait up to 10sec trying to umount. If the resource stays
        busy the call will raise an exception unless raise_on_busy
        is set to False. In case the umount failed and raise_on_busy
        is set to False, the method returns False to indicate the
        error condition.

        :return: True or False

        :rtype: bool
        """
        if self.is_mounted():
            umounted_successfully = False
            for busy in range(0, 10):
                try:
                    Command.run(['umount', self.mountpoint])
                    umounted_successfully = True
                    break
                except Exception:
                    log.warning(
                        '%d umount of %s failed, try again in 1sec',
                        busy, self.mountpoint
                    )
                    time.sleep(1)
            if not umounted_successfully:
                if raise_on_busy:
                    lsof = Path.which('lsof', access_mode=os.X_OK)
                    if lsof:
                        open_files = Command.run(
                            [lsof, '+c', '0', self.mountpoint],
                            raise_on_error=False
                        )
                        open_files_info = 'Open files status:{0}{1}'.format(
                            os.linesep, open_files.output
                        )
                    else:
                        open_files_info = 'For further details install: lsof'
                    message = dedent('''\n
                        Failed to umount: {0}.

                        Your build host system is in an inconsistent state.
                        The cleanup of the created resource was not possible
                        because it is still busy. This resource and all nested
                        resources stays active on your host and needs a manual
                        cleanup.

                        Please do not use the intermediate state of the image
                        files created so far. There is no guarantee that the
                        produced results are valid.

                        {1}
                    ''')
                    raise KiwiUmountBusyError(
                        message.format(self.mountpoint, open_files_info)
                    )
                else:
                    log.warning(
                        '{0} still busy at {1}'.format(
                            self.mountpoint, type(self).__name__
                        )
                    )
                    # skip removing the mountpoint directory
                    return False
        return True

    def is_mounted(self) -> bool:
        """
        Check if mounted

        :return: True or False

        :rtype: bool
        """
        mountpoint_call = Command.run(
            command=['mountpoint', '-q', self.mountpoint],
            raise_on_error=False
        )
        if mountpoint_call.returncode == 0:
            return True
        else:
            return False



# Copyright (c) 2018 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import importlib
from abc import (
    ABCMeta,
    abstractmethod
)

# project
from kiwi.runtime_config import RuntimeConfig
from kiwi.exceptions import (
    KiwiOCIArchiveToolError
)


class OCI(metaclass=ABCMeta):
    """
    **OCI Factory**
    """
    @abstractmethod
    def __init__(self) -> None:
        return None  # pragma: no cover

    @staticmethod
    def new():
        name_map = {
            'umoci': 'Umoci',
            'buildah': 'Buildah'
        }
        runtime_config = RuntimeConfig()
        tool_name = runtime_config.get_oci_archive_tool()
        try:
            oci_tool = importlib.import_module(
                'kiwi.oci_tools.{}'.format(tool_name)
            )
            module_name = 'OCI{}'.format(name_map[tool_name])
            return oci_tool.__dict__[module_name]()
        except Exception:
            raise KiwiOCIArchiveToolError(
                'No support for {0} tool available'.format(tool_name)
            )



from mock import patch

from kiwi.markup import Markup

from kiwi.exceptions import KiwiAnyMarkupPluginError


class TestMarkup:
    @patch('kiwi.markup.any.MarkupAny')
    def test_MarkupAny(self, mock_MarkupAny):
        Markup.new('description')
        mock_MarkupAny.assert_called_once_with('description')

    @patch('kiwi.markup.xml.MarkupXML')
    @patch('kiwi.markup.any.MarkupAny')
    def test_MarkupXML(self, mock_MarkupAny, mock_MarkupXML):
        mock_MarkupAny.side_effect = KiwiAnyMarkupPluginError('load-error')
        Markup.new('description')
        mock_MarkupXML.assert_called_once_with('description')



import subprocess
import logging
from dataclasses import dataclass
from os import getenv, path
from typing import Any, List, Optional, Tuple

import pytest
import testinfra


CONTAINER_URL = "registry.opensuse.org/opensuse/tumbleweed:latest"
SELECTED_CONTAINER_RUNTIME: Optional[str] = None


@dataclass(frozen=True)
class Container:
    name: str
    url: str

    def launch_and_prepare_container(
        self, runner_binary: str
    ) -> Tuple[Any, str]:
        container_id = (
            subprocess.check_output(
                [runner_binary, "run", "-d", "-it", self.url, "/bin/sh"]
            )
            .decode()
            .strip()
        )
        subprocess.check_call(
            [
                runner_binary,
                "cp",
                path.abspath(
                    path.join(
                        path.dirname(__file__),
                        "..",
                        "..",
                        "kiwi",
                        "config",
                        "functions.sh",
                    )
                ),
                container_id + ":" + "/bin/functions.sh",
            ],
        )

        return (
            testinfra.get_host(f"{runner_binary}://{container_id}"),
            container_id,
        )

    @staticmethod
    def cleanup_container(runner_binary: str, container_id: str) -> None:
        subprocess.check_call([runner_binary, "rm", "-f", container_id])


TUMBLEWEED_CONTAINER = Container(
    name="Tumbleweed",
    url="registry.opensuse.org/opensuse/tumbleweed:latest",
)

CONTAINERS = [
    Container(
        name="Leap-15.3", url="registry.opensuse.org/opensuse/leap:15.3"
    ),
    Container(
        name="Leap-15.2", url="registry.opensuse.org/opensuse/leap:15.2"
    ),
    TUMBLEWEED_CONTAINER,
    Container(name="SLE-15-SP3", url="registry.suse.com/suse/sle15:15.3"),
    Container(name="SLE-15-SP2", url="registry.suse.com/suse/sle15:15.2"),
    Container(name="SLE-15-SP1", url="registry.suse.com/suse/sle15:15.1"),
    Container(
        name="SLE-12-SP5", url="registry.suse.com/suse/sles12sp5:latest"
    ),
    Container(name="centos:stream8", url="quay.io/centos/centos:stream8"),
]


@pytest.fixture(autouse=True, scope="session")
def _check_container_runtimes():
    """Fixture that checks whether the podman & docker runtimes work and
    selects one (either the only one that works or the one from the environment
    variable `CONTAINER_RUNTIME`).
    """
    localhost = testinfra.host.get_host("local://")

    runtimes = ["podman", "docker"]
    working_container_runtimes: List[str] = []

    for runtime in runtimes:
        if not localhost.exists(runtime):
            continue
        try:
            con, con_id = TUMBLEWEED_CONTAINER.launch_and_prepare_container(
                runtime
            )
            assert con.file("/etc/os-release").exists
            Container.cleanup_container(runtime, con_id)

            working_container_runtimes.append(runtime)
        except Exception as issue:
            logging.debug(issue)

    if len(working_container_runtimes) == 0:
        raise RuntimeError("No working container runtime found")
    elif len(working_container_runtimes) == 1:
        global SELECTED_CONTAINER_RUNTIME
        SELECTED_CONTAINER_RUNTIME = working_container_runtimes[0]
    else:
        assert len(working_container_runtimes) == 2

        runtime_choice = getenv("CONTAINER_RUNTIME", "podman").lower()
        if runtime_choice not in runtimes:
            raise ValueError(f"Invalid CONTAINER_RUNTIME {runtime_choice}")

        SELECTED_CONTAINER_RUNTIME = runtime_choice


def get_container_by_name(container_name: str = "Tumbleweed") -> Container:
    match = [c for c in CONTAINERS if c.name == container_name]
    assert len(match) == 1, "found {0} containers with the name {1}".format(
        len(match), container_name
    )

    return match[0]


def get_container_name(request) -> str:
    return getattr(request, "param", "Tumbleweed")


@pytest.fixture(scope="function")
def container_per_test(request, _check_container_runtimes):
    """Fixture that requests a new container for each function.

    By default the Tumbleweed container is used if no parameter is passed to
    this fixture.

    This fixture is pretty expensive, as it will launch and destroy a new
    container for **every** test function. Thus only use it for tests that
    perform extensive mutation, which you do not feel comfortable to undo
    quickly in the test.
    """
    assert SELECTED_CONTAINER_RUNTIME is not None
    container = get_container_by_name(get_container_name(request))
    con, container_id = container.launch_and_prepare_container(
        SELECTED_CONTAINER_RUNTIME
    )
    yield con
    Container.cleanup_container(SELECTED_CONTAINER_RUNTIME, container_id)


@pytest.fixture(scope="session")
def shared_container(request, _check_container_runtimes):
    """Fixture that requests a new container for the whole test session.

    By default the Tumbleweed container is used if no parameter is passed to
    this fixture.

    **Caution:** The container will be shared by all tests that request this
    fixture. You **must** ensure that you do not mutate the container in your
    test, as later tests could then fail due to your changes.
    """
    assert SELECTED_CONTAINER_RUNTIME is not None
    container = get_container_by_name(get_container_name(request))
    con, container_id = container.launch_and_prepare_container(
        SELECTED_CONTAINER_RUNTIME
    )
    yield con
    Container.cleanup_container(SELECTED_CONTAINER_RUNTIME, container_id)



from mock import patch

from kiwi.utils.block import BlockID


class TestBlockID:
    def setup(self):
        self.blkid = BlockID('device')

    @patch('kiwi.utils.block.Command.run')
    def test_setup_with_uuid_format(self, mock_command):
        BlockID('UUID=uuid')
        mock_command.assert_called_once_with(
            ['blkid', '--uuid', 'uuid']
        )

    @patch('kiwi.utils.block.Command.run')
    def test_get_blkid(self, mock_command):
        self.blkid.get_blkid('LABEL')
        mock_command.assert_called_once_with(
            ['blkid', 'device', '-s', 'LABEL', '-o', 'value']
        )

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_filesystem(self, mock_get_blkid):
        self.blkid.get_filesystem()
        mock_get_blkid.assert_called_once_with('TYPE')

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_label(self, mock_get_blkid):
        self.blkid.get_label()
        mock_get_blkid.assert_called_once_with('LABEL')

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_uuid(self, mock_get_blkid):
        self.blkid.get_uuid()
        mock_get_blkid.assert_called_once_with('UUID')



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>

# project
from kiwi.filesystem.base import FileSystemBase
from kiwi.command import Command


class FileSystemSwap(FileSystemBase):
    """
    **Implements creation of swap space**
    """
    def create_on_device(self, label: str = None):
        """
        Create swap space on block device

        :param string label: label name
        """
        device = self.device_provider.get_device()
        if label:
            self.custom_args['create_options'].append('-L')
            self.custom_args['create_options'].append(label)
        Command.run(
            ['mkswap'] + self.custom_args['create_options'] + [device]
        )



import logging
from mock import (
    patch, Mock
)
from pytest import (
    raises, fixture
)

from kiwi.system.uri import Uri
from kiwi.system.root_import.oci import RootImportOCI

from kiwi.exceptions import KiwiRootImportError


class TestRootImportOCI:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    @patch('os.path.exists')
    def setup(self, mock_path):
        mock_path.return_value = True
        with patch.dict('os.environ', {'HOME': '../data'}):
            self.oci_import = RootImportOCI(
                'root_dir', Uri('file:///image.tar'),
                {'archive_transport': 'oci-archive'}
            )
        assert self.oci_import.image_file == '/image.tar'

    @patch('os.path.exists')
    def test_failed_init(self, mock_path):
        mock_path.return_value = False
        with raises(KiwiRootImportError):
            RootImportOCI(
                'root_dir', Uri('file:///image.tar.xz'),
                {'archive_transport': 'oci-archive'}
            )

    @patch('kiwi.system.root_import.oci.Compress')
    @patch('kiwi.system.root_import.base.Checksum')
    @patch('kiwi.system.root_import.oci.Path.create')
    @patch('kiwi.system.root_import.oci.OCI')
    def test_sync_data(self, mock_OCI, mock_path, mock_md5, mock_compress):
        oci = Mock()
        mock_OCI.new.return_value = oci
        md5 = Mock()
        mock_md5.return_value = Mock()

        uncompress = Mock()
        uncompress.get_format = Mock(return_value=None)
        mock_compress.return_value = uncompress

        self.oci_import.sync_data()

        mock_OCI.new.assert_called_once_with()

        oci.unpack.assert_called_once_with()
        oci.import_rootfs.assert_called_once_with(
            'root_dir'
        )
        mock_md5.assert_called_once_with('root_dir/image/imported_root')
        md5.md5.called_once_with('root_dir/image/imported_root.md5')
        uncompress.get_format.assert_called_once_with()

    @patch('kiwi.system.root_import.oci.Compress')
    @patch('kiwi.system.root_import.base.Checksum')
    @patch('kiwi.system.root_import.oci.Path.create')
    @patch('kiwi.system.root_import.oci.OCI')
    def test_sync_data_compressed_image(
        self, mock_OCI, mock_path, mock_md5, mock_compress
    ):
        oci = Mock()
        mock_OCI.new.return_value = oci
        md5 = Mock()
        mock_md5.return_value = Mock()

        uncompress = Mock()
        uncompress.get_format = Mock(return_value='xz')
        mock_compress.return_value = uncompress

        self.oci_import.sync_data()

        mock_OCI.new.assert_called_once_with()

        oci.unpack.assert_called_once_with()
        oci.import_rootfs.assert_called_once_with(
            'root_dir'
        )
        mock_md5.assert_called_once_with('root_dir/image/imported_root')
        md5.md5.called_once_with('root_dir/image/imported_root.md5')
        uncompress.get_format.assert_called_once_with()
        uncompress.uncompress.assert_called_once_with(True)

    @patch('os.path.exists')
    @patch('kiwi.system.root_import.base.Checksum')
    @patch('kiwi.system.root_import.oci.Path.create')
    @patch('kiwi.system.root_import.oci.OCI')
    def test_sync_data_unknown_uri(
        self, mock_OCI, mock_path, mock_md5, mock_exists
    ):
        mock_exists.return_value = True
        oci = Mock()
        mock_OCI.new.return_value = oci
        md5 = Mock()
        mock_md5.return_value = Mock()
        with patch.dict('os.environ', {'HOME': '../data'}):
            oci_import = RootImportOCI(
                'root_dir', Uri('docker:image:tag'),
                {'archive_transport': 'docker-archive'}
            )

        with self._caplog.at_level(logging.WARNING):
            oci_import.sync_data()
            mock_OCI.new.assert_called_once_with()
            oci.import_container_image.assert_called_once_with(
                'docker:image:tag'
            )
            oci.unpack.assert_called_once_with()
            oci.import_rootfs.assert_called_once_with(
                'root_dir'
            )
            mock_md5.assert_called_once_with('root_dir/image/imported_root')
            md5.md5.called_once_with('root_dir/image/imported_root.md5')



import logging
from pytest import fixture
from mock import (
    patch, mock_open
)

import mock

import kiwi

from kiwi.container.oci import ContainerImageOCI
from kiwi.version import __version__


class TestContainerImageOCI:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    @patch('kiwi.oci_tools.umoci.CommandCapabilities.has_option_in_help')
    def setup(self, mock_cmd_caps):
        mock_cmd_caps.return_value = True
        self.runtime_config = mock.Mock()
        self.runtime_config.get_container_compression = mock.Mock(
            return_value=None
        )
        kiwi.container.oci.RuntimeConfig = mock.Mock(
            return_value=self.runtime_config
        )
        self.oci = ContainerImageOCI(
            'root_dir', 'docker-archive', {
                'container_name': 'foo/bar',
                'additional_tags': ['current', 'foobar']
            }
        )

    @patch('kiwi.oci_tools.umoci.CommandCapabilities.has_option_in_help')
    def test_init_custom_args(self, mock_cmd_caps):
        mock_cmd_caps.return_value = True
        custom_args = {
            'container_name': 'foo',
            'container_tag': '1.0',
            'additional_tags': ['current', 'foobar'],
            'entry_command': ['/bin/bash', '-x'],
            'entry_subcommand': ['ls', '-l'],
            'maintainer': 'tux',
            'user': 'root',
            'workingdir': '/root',
            'expose_ports': ['80', '42'],
            'volumes': ['/var/log', '/tmp'],
            'environment': {'PATH': '/bin', 'FOO': 'bar'},
            'labels': {'a': 'value', 'b': 'value'}
        }
        container = ContainerImageOCI(
            'root_dir', 'oci-archive', custom_args
        )
        assert container.oci_config == custom_args

    @patch('kiwi.oci_tools.umoci.CommandCapabilities.has_option_in_help')
    def test_init_without_custom_args(self, mock_cmd_caps):
        mock_cmd_caps.return_value = True
        container = ContainerImageOCI('root_dir', 'oci-archive')
        assert container.oci_config == {
            'container_name': 'kiwi-container',
            'container_tag': 'latest',
            'history': {'created_by': 'KIWI {0}'.format(__version__)}
        }

    @patch('kiwi.container.oci.RuntimeConfig')
    @patch('kiwi.oci_tools.umoci.CommandCapabilities.has_option_in_help')
    @patch('kiwi.defaults.Defaults.is_buildservice_worker')
    def test_init_in_buildservice(
        self, mock_buildservice, mock_cmd_caps, mock_RuntimeConfig
    ):
        mock_buildservice.return_value = True
        mock_cmd_caps.return_value = True

        m_open = mock_open()
        with patch('builtins.open', m_open, create=True):
            m_open.return_value.__iter__ = lambda _:\
                iter(['BUILD_DISTURL=obs://build.opensuse.org/some:project'])
            container = ContainerImageOCI(
                'root_dir', 'oci-archive'
            )

        m_open.assert_called_once_with('/.buildenv')
        assert container.oci_config['labels'] == {
            'org.openbuildservice.disturl':
            'obs://build.opensuse.org/some:project'
        }

        m_open = mock_open()
        with patch('builtins.open', m_open, create=True):
            m_open.return_value.__iter__ = lambda _:\
                iter(['BUILD_DISTURL=obs://build.opensuse.org/some:project'])
            container = ContainerImageOCI(
                'root_dir', 'oci-archive', {'labels': {'label': 'value'}}
            )

        m_open.assert_called_once_with('/.buildenv')
        assert container.oci_config['labels'] == {
            'label': 'value',
            'org.openbuildservice.disturl':
            'obs://build.opensuse.org/some:project'
        }

    @patch('kiwi.container.oci.RuntimeConfig')
    @patch('kiwi.oci_tools.umoci.CommandCapabilities.has_option_in_help')
    @patch('kiwi.defaults.Defaults.is_buildservice_worker')
    def test_init_in_buildservice_without_disturl(
        self, mock_buildservice, mock_cmd_caps, mock_RuntimeConfig
    ):
        mock_buildservice.return_value = True
        mock_cmd_caps.return_value = True

        m_open = mock_open()
        with patch('builtins.open', m_open, create=True):
            m_open.return_value.__iter__ = lambda _: iter(['line content'])
            with self._caplog.at_level(logging.WARNING):
                container = ContainerImageOCI('root_dir', 'oci-archive')

        m_open.assert_called_once_with('/.buildenv')
        assert 'labels' not in container.oci_config

    @patch('kiwi.container.oci.OCI')
    @patch('kiwi.container.oci.Defaults.get_shared_cache_location')
    def test_create_oci_archive(self, mock_cache, mock_OCI):
        mock_cache.return_value = 'var/cache/kiwi'
        mock_oci = mock.Mock()
        mock_OCI.new.return_value = mock_oci

        self.oci.archive_transport = 'oci-archive'
        self.oci.create('result.tar', None)

        mock_oci.init_container.assert_called_once_with()
        mock_oci.unpack.assert_called_once_with()
        mock_oci.sync_rootfs.assert_called_once_with(
            'root_dir', [
                'image', '.profile', '.kconfig', 'run/*', 'tmp/*',
                '.buildenv', 'var/cache/kiwi', 'dev/*', 'sys/*', 'proc/*'
            ]
        )
        mock_oci.repack.assert_called_once_with({
            'container_name': 'foo/bar',
            'additional_tags': ['current', 'foobar'],
            'container_tag': 'latest',
            'entry_subcommand': ['/bin/bash'],
            'history': {'created_by': 'KIWI {0}'.format(__version__)}
        })
        mock_oci.set_config.assert_called_once_with({
            'container_name': 'foo/bar',
            'additional_tags': ['current', 'foobar'],
            'container_tag': 'latest',
            'entry_subcommand': ['/bin/bash'],
            'history': {'created_by': 'KIWI {0}'.format(__version__)}
        })
        mock_oci.post_process.assert_called_once_with()
        mock_oci.export_container_image.assert_called_once_with(
            'result.tar', 'oci-archive', 'latest', []
        )

    @patch('kiwi.container.oci.Compress')
    @patch('kiwi.container.oci.OCI')
    @patch('kiwi.container.oci.Defaults.get_shared_cache_location')
    def test_create_derived_docker_archive(
        self, mock_cache, mock_OCI, mock_compress
    ):
        mock_cache.return_value = 'var/cache/kiwi'
        mock_oci = mock.Mock()
        mock_OCI.new.return_value = mock_oci

        self.runtime_config.get_container_compression.return_value = 'xz'

        self.oci.create('result.tar', 'root_dir/image/image_file')

        mock_oci.import_container_image.assert_called_once_with(
            'oci-archive:root_dir/image/image_file:base_layer'
        )
        mock_oci.unpack.assert_called_once_with()
        mock_oci.sync_rootfs.assert_called_once_with(
            'root_dir', [
                'image', '.profile', '.kconfig', 'run/*', 'tmp/*',
                '.buildenv', 'var/cache/kiwi', 'dev/*', 'sys/*', 'proc/*'
            ]
        )
        mock_oci.repack.assert_called_once_with({
            'container_name': 'foo/bar',
            'additional_tags': ['current', 'foobar'],
            'container_tag': 'latest',
            'history': {'created_by': 'KIWI {0}'.format(__version__)}
        })
        mock_oci.set_config.assert_called_once_with({
            'container_name': 'foo/bar',
            'additional_tags': ['current', 'foobar'],
            'container_tag': 'latest',
            'history': {'created_by': 'KIWI {0}'.format(__version__)}
        })
        mock_oci.post_process.assert_called_once_with()
        mock_oci.export_container_image.assert_called_once_with(
            'result.tar', 'docker-archive', 'foo/bar:latest',
            ['foo/bar:current', 'foo/bar:foobar']
        )

        mock_compress.assert_called_once_with('result.tar')



import logging
from mock import patch
from pytest import (
    raises, fixture
)

from kiwi.storage.loop_device import LoopDevice

from kiwi.exceptions import KiwiLoopSetupError


class TestLoopDevice:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    @patch('os.path.exists')
    def setup(self, mock_exists):
        mock_exists.return_value = False
        self.loop = LoopDevice('loop-file', 20, 4096)

    def test_loop_setup_invalid(self):
        with raises(KiwiLoopSetupError):
            LoopDevice('loop-file-does-not-exist-and-no-size-given')

    def test_get_device(self):
        assert self.loop.get_device() == ''

    def test_is_loop(self):
        assert self.loop.is_loop() is True

    @patch('os.path.exists')
    @patch('kiwi.storage.loop_device.Command.run')
    @patch('kiwi.storage.loop_device.CommandCapabilities.has_option_in_help')
    def test_create(
        self, mock_has_option_in_help, mock_command, mock_exists
    ):
        mock_has_option_in_help.return_value = True
        mock_exists.return_value = False
        self.loop.create()
        call = mock_command.call_args_list[0]
        assert mock_command.call_args_list[0] == \
            call([
                'qemu-img', 'create', 'loop-file', '20M'
            ])
        call = mock_command.call_args_list[1]
        assert mock_command.call_args_list[1] == \
            call([
                'losetup', '--sector-size', '4096',
                '-f', '--show', 'loop-file'
            ])
        mock_has_option_in_help.return_value = False
        mock_command.reset_mock()
        self.loop.create()
        assert mock_command.call_args_list[1] == \
            call([
                'losetup', '--logical-blocksize', '4096',
                '-f', '--show', 'loop-file'
            ])
        self.loop.node_name = None

    @patch('kiwi.storage.loop_device.Command.run')
    def test_destructor(self, mock_command):
        self.loop.node_name = '/dev/loop0'
        mock_command.side_effect = Exception
        self.loop.__del__()
        with self._caplog.at_level(logging.WARNING):
            mock_command.assert_called_once_with(
                ['losetup', '-d', '/dev/loop0']
            )
        self.loop.node_name = None



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import importlib
from abc import (
    ABCMeta,
    abstractmethod
)
from typing import Dict

# project
from kiwi.exceptions import (
    KiwiContainerSetupError
)


class ContainerSetup(metaclass=ABCMeta):
    """
        container setup factory
    """
    @abstractmethod
    def __init__(self) -> None:
        return None  # pragma: no cover

    @staticmethod
    def new(name: str, root_dir: str, custom_args: Dict=None):  # noqa: E252
        name_map = {
            'docker': 'Docker',
            'oci': 'OCI',
            'appx': 'Appx'
        }
        try:
            container_setup = importlib.import_module(
                'kiwi.container.setup.{}'.format(name)
            )
            module_name = 'ContainerSetup{}'.format(name_map[name])
            return container_setup.__dict__[module_name](root_dir, custom_args)
        except Exception:
            raise KiwiContainerSetupError(
                'Support for {0} container not implemented'.format(name)
            )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import logging
from collections import OrderedDict
from typing import (
    Dict, NamedTuple
)

# project
from kiwi.utils.temporary import Temporary
from kiwi.command import Command
from kiwi.storage.device_provider import DeviceProvider
from kiwi.storage.mapped_device import MappedDevice
from kiwi.partitioner import Partitioner
from kiwi.exceptions import KiwiCustomPartitionConflictError

ptable_entry_type = NamedTuple(
    'ptable_entry_type', [
        ('mbsize', int),
        ('partition_name', str),
        ('partition_type', str),
        ('mountpoint', str),
        ('filesystem', str)
    ]
)

log = logging.getLogger('kiwi')


class Disk(DeviceProvider):
    """
    **Implements storage disk and partition table setup**

    :param string table_type: Partition table type name
    :param object storage_provider: Instance of class based on DeviceProvider
    :param int start_sector: sector number
    """
    def __init__(
        self, table_type: str, storage_provider: DeviceProvider,
        start_sector: int = None
    ):
        # bind the underlaying block device providing class instance
        # to this object (e.g loop) if present. This is done to guarantee
        # the correct destructor order when the device should be released.
        self.storage_provider = storage_provider

        # list of protected map ids. If used in a custom partitions
        # setup this will lead to a raise conditition in order to
        # avoid conflicts with the existing partition layout and its
        # customizaton capabilities
        self.protected_map_ids = [
            'root',
            'readonly',
            'boot',
            'prep',
            'spare',
            'swap',
            'efi_csm',
            'efi'
        ]

        self.partition_map: Dict[str, str] = {}
        self.public_partition_id_map: Dict[str, str] = {}
        self.partition_id_map: Dict[str, str] = {}
        self.is_mapped = False

        self.partitioner = Partitioner.new(
            table_type, storage_provider, start_sector
        )

        self.table_type = table_type

    def get_device(self) -> Dict[str, MappedDevice]:
        """
        Names of partition devices

        Note that the mapping requires an explicit map() call

        :return: instances of MappedDevice

        :rtype: dict
        """
        device_map = {}
        for partition_name, device_node in list(self.partition_map.items()):
            device_map[partition_name] = MappedDevice(
                device=device_node, device_provider=self
            )
        return device_map

    def is_loop(self) -> bool:
        """
        Check if storage provider is loop based

        The information is taken from the storage provider. If
        the storage provider is loop based the disk is it too

        :return: True or False

        :rtype: bool
        """
        return self.storage_provider.is_loop()

    def create_custom_partitions(
        self, table_entries: Dict[str, ptable_entry_type]
    ) -> None:
        """
        Create partitions from custom data set

        .. code:: python

           table_entries = {
               map_name: ptable_entry_type
           }

        :param dict table: partition table spec
        """
        for map_name in table_entries:
            if map_name in self.protected_map_ids:
                raise KiwiCustomPartitionConflictError(
                    f'Cannot use reserved table entry name: {map_name!r}'
                )
            id_name = f'kiwi_{map_name.title()}Part'
            entry = table_entries[map_name]
            self.partitioner.create(
                entry.partition_name, entry.mbsize, entry.partition_type
            )
            self._add_to_map(map_name)
            self._add_to_public_id_map(id_name)

    def create_root_partition(self, mbsize: str):
        """
        Create root partition

        Populates kiwi_RootPart(id) and kiwi_BootPart(id) if no extra
        boot partition is requested

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.lxroot', mbsize, 't.linux')
        self._add_to_map('root')
        self._add_to_public_id_map('kiwi_RootPart')
        if 'kiwi_ROPart' in self.public_partition_id_map:
            self._add_to_public_id_map('kiwi_RWPart')
        if 'kiwi_BootPart' not in self.public_partition_id_map:
            self._add_to_public_id_map('kiwi_BootPart')

    def create_root_lvm_partition(self, mbsize: str):
        """
        Create root partition for use with LVM

        Populates kiwi_RootPart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.lxlvm', mbsize, 't.lvm')
        self._add_to_map('root')
        self._add_to_public_id_map('kiwi_RootPart')

    def create_root_raid_partition(self, mbsize: str):
        """
        Create root partition for use with MD Raid

        Populates kiwi_RootPart(id) and kiwi_RaidPart(id) as well
        as the default raid device node at boot time which is
        configured to be kiwi_RaidDev(/dev/mdX)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.lxraid', mbsize, 't.raid')
        self._add_to_map('root')
        self._add_to_public_id_map('kiwi_RootPart')
        self._add_to_public_id_map('kiwi_RaidPart')

    def create_root_readonly_partition(self, mbsize: str):
        """
        Create root readonly partition for use with overlayfs

        Populates kiwi_ReadOnlyPart(id), the partition is meant to
        contain a squashfs readonly filesystem. The partition size
        should be the size of the squashfs filesystem in order to
        avoid wasting disk space

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.lxreadonly', mbsize, 't.linux')
        self._add_to_map('readonly')
        self._add_to_public_id_map('kiwi_ROPart')

    def create_boot_partition(self, mbsize: str):
        """
        Create boot partition

        Populates kiwi_BootPart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.lxboot', mbsize, 't.linux')
        self._add_to_map('boot')
        self._add_to_public_id_map('kiwi_BootPart')

    def create_prep_partition(self, mbsize: str):
        """
        Create prep partition

        Populates kiwi_PrepPart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.prep', mbsize, 't.prep')
        self._add_to_map('prep')
        self._add_to_public_id_map('kiwi_PrepPart')

    def create_spare_partition(self, mbsize: str):
        """
        Create spare partition for custom use

        Populates kiwi_SparePart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.spare', mbsize, 't.linux')
        self._add_to_map('spare')
        self._add_to_public_id_map('kiwi_SparePart')

    def create_swap_partition(self, mbsize: str):
        """
        Create swap partition

        Populates kiwi_SwapPart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.swap', mbsize, 't.swap')
        self._add_to_map('swap')
        self._add_to_public_id_map('kiwi_SwapPart')

    def create_efi_csm_partition(self, mbsize: str):
        """
        Create EFI bios grub partition

        Populates kiwi_BiosGrub(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.legacy', mbsize, 't.csm')
        self._add_to_map('efi_csm')
        self._add_to_public_id_map('kiwi_BiosGrub')

    def create_efi_partition(self, mbsize: str):
        """
        Create EFI partition

        Populates kiwi_EfiPart(id)

        :param str mbsize: partition size NumberString or 'all_free'
        """
        self.partitioner.create('p.UEFI', mbsize, 't.efi')
        self._add_to_map('efi')
        self._add_to_public_id_map('kiwi_EfiPart')

    def activate_boot_partition(self):
        """
        Activate boot partition

        Note: not all Partitioner instances supports this
        """
        partition_id = None
        if 'prep' in self.partition_id_map:
            partition_id = self.partition_id_map['prep']
        elif 'boot' in self.partition_id_map:
            partition_id = self.partition_id_map['boot']
        elif 'root' in self.partition_id_map:
            partition_id = self.partition_id_map['root']

        if partition_id:
            self.partitioner.set_flag(partition_id, 'f.active')

    def create_hybrid_mbr(self):
        """
        Turn partition table into a hybrid GPT/MBR table

        Note: only GPT tables supports this
        """
        self.partitioner.set_hybrid_mbr()

    def create_mbr(self):
        """
        Turn partition table into MBR (msdos table)

        Note: only GPT tables supports this
        """
        self.partitioner.set_mbr()

    def wipe(self):
        """
        Zap (destroy) any GPT and MBR data structures if present
        For DASD disks create a new VTOC table
        """
        if 'dasd' in self.table_type:
            log.debug('Initialize DASD disk with new VTOC table')
            fdasd_input = Temporary().new_file()
            with open(fdasd_input.name, 'w') as vtoc:
                vtoc.write('y\n\nw\nq\n')
            bash_command = ' '.join(
                [
                    'cat', fdasd_input.name, '|',
                    'fdasd', '-f', self.storage_provider.get_device()
                ]
            )
            try:
                Command.run(
                    ['bash', '-c', bash_command]
                )
            except Exception:
                # unfortunately fdasd reports that it can't read in the
                # partition table which I consider a bug in fdasd. However
                # the table was correctly created and therefore we continue.
                # Problem is that we are not able to detect real errors
                # with the fdasd operation at that point.
                log.debug('potential fdasd errors were ignored')
        else:
            log.debug('Initialize %s disk', self.table_type)
            Command.run(
                [
                    'sgdisk', '--zap-all', self.storage_provider.get_device()
                ]
            )

    def map_partitions(self):
        """
        Map/Activate partitions

        In order to access the partitions through a device node it is
        required to map them if the storage provider is loop based
        """
        if self.storage_provider.is_loop():
            Command.run(
                ['kpartx', '-s', '-a', self.storage_provider.get_device()]
            )
            self.is_mapped = True
        else:
            Command.run(
                ['partprobe', self.storage_provider.get_device()]
            )

    def get_public_partition_id_map(self) -> Dict[str, str]:
        """
        Populated partition name to number map
        """
        return OrderedDict(
            sorted(self.public_partition_id_map.items())
        )

    def _add_to_public_id_map(self, name, value=None):
        if not value:
            value = self.partitioner.get_id()
        self.public_partition_id_map[name] = value

    def _add_to_map(self, name):
        device_node = None
        partition_number = format(self.partitioner.get_id())
        if self.storage_provider.is_loop():
            device_base = os.path.basename(self.storage_provider.get_device())
            device_node = ''.join(
                ['/dev/mapper/', device_base, 'p', partition_number]
            )
        else:
            device = self.storage_provider.get_device()
            if device[-1].isdigit():
                device_node = ''.join(
                    [device, 'p', partition_number]
                )
            else:
                device_node = ''.join(
                    [device, partition_number]
                )
        if device_node:
            self.partition_map[name] = device_node
            self.partition_id_map[name] = partition_number

    def __del__(self):
        if self.storage_provider.is_loop() and self.is_mapped:
            log.info('Cleaning up %s instance', type(self).__name__)
            try:
                for device_node in self.partition_map.values():
                    Command.run(['dmsetup', 'remove', device_node])
            except Exception:
                log.warning(
                    'cleanup of partition device maps failed, %s still busy',
                    self.storage_provider.get_device()
                )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import logging
from kiwi.utils.temporary import Temporary
from typing import (
    Optional, List
)

# project
from kiwi.defaults import Defaults
from kiwi.system.identifier import SystemIdentifier
from kiwi.utils.sync import DataSync
from kiwi.system.prepare import SystemPrepare
from kiwi.system.profile import Profile
from kiwi.system.setup import SystemSetup
from kiwi.archive.cpio import ArchiveCpio
from kiwi.utils.compress import Compress
from kiwi.path import Path
from kiwi.boot.image.base import BootImageBase

log = logging.getLogger('kiwi')


class BootImageKiwi(BootImageBase):
    """
    **Implements preparation and creation of kiwi boot(initrd) images**

    The kiwi initrd is a customized first boot initrd which allows
    to control the first boot an appliance. The kiwi initrd replaces
    itself after first boot by the result of dracut.
    """
    @staticmethod
    def has_initrd_support() -> bool:
        """
        This instance supports initrd preparation and creation
        """
        return True

    def post_init(self) -> None:
        """
        Post initialization method

        Creates custom directory to prepare the boot image
        root filesystem which is a separate image to create
        the initrd from
        """
        # builtin kiwi initrd builds its own root tree to create
        # an initrd from. Thus there is no pre defined boot root
        # directory
        self.boot_root_directory = ''

        self.temp_directories: List[str] = []
        self.load_boot_xml_description()

    def prepare(self) -> None:
        """
        Prepare new root system suitable to create a kiwi initrd from it
        """
        if self.boot_xml_state:
            self.boot_root_directory_temporary = Temporary(
                prefix='kiwi_boot_root.', path=self.target_dir
            ).new_dir()
            self.boot_root_directory = self.boot_root_directory_temporary.name
            self.temp_directories.append(
                self.boot_root_directory
            )
            boot_image_name = self.boot_xml_state.xml_data.get_name()

            self.import_system_description_elements()

            log.info('Preparing boot image')
            system = SystemPrepare(
                xml_state=self.boot_xml_state,
                root_dir=self.boot_root_directory,
                allow_existing=True
            )
            manager = system.setup_repositories(
                signing_keys=self.signing_keys, target_arch=self.target_arch
            )
            system.install_bootstrap(
                manager
            )
            system.install_system(
                manager
            )

            profile = Profile(self.boot_xml_state)
            profile.add('kiwi_initrdname', boot_image_name)

            defaults = Defaults()
            defaults.to_profile(profile)

            self.setup = SystemSetup(
                self.boot_xml_state, self.boot_root_directory
            )
            profile.create(
                Defaults.get_profile_file(self.boot_root_directory)
            )
            self.setup.import_description()
            self.setup.import_overlay_files(
                follow_links=True
            )
            self.setup.call_config_script()

            system.pinch_system(
                manager=manager, force=True
            )
            # make sure system instance is cleaned up before setting up
            del system

            self.setup.call_image_script()
            self.setup.create_init_link_from_linuxrc()

    def create_initrd(
        self, mbrid: Optional[SystemIdentifier] = None,
        basename: Optional[str] = None, install_initrd: bool = False
    ) -> None:
        """
        Create initrd from prepared boot system tree and compress the result

        :param SystemIdentifier mbrid: instance of ImageIdentifier
        :param str basename: base initrd file name
        :param bool install_initrd: installation media initrd

        """
        if self.is_prepared():
            log.info('Creating initrd cpio archive')
            # we can't simply exclude boot when building the archive
            # because the file boot/mbrid must be preserved. Because of
            # that we create a copy of the boot directory and remove
            # everything in boot/ except for boot/mbrid. The original
            # boot directory should not be changed because we rely
            # on other data in boot/ e.g the kernel to be available
            # for the entire image building process
            if basename:
                kiwi_initrd_basename = basename
            else:
                kiwi_initrd_basename = self.initrd_base_name
            temp_boot_root = Temporary(
                prefix='kiwi_boot_root_copy.'
            ).new_dir()
            temp_boot_root_directory = temp_boot_root.name
            os.chmod(temp_boot_root_directory, 0o755)
            data = DataSync(
                self.boot_root_directory + '/',
                temp_boot_root_directory
            )
            data.sync_data(
                options=['-a']
            )
            boot_directory = temp_boot_root_directory + '/boot'
            Path.wipe(boot_directory)
            if mbrid:
                log.info(
                    '--> Importing mbrid: %s', mbrid.get_id()
                )
                Path.create(boot_directory)
                image_identifier = boot_directory + '/mbrid'
                mbrid.write(image_identifier)

            cpio = ArchiveCpio(
                os.sep.join([self.target_dir, kiwi_initrd_basename])
            )
            # the following is a list of directories which were needed
            # during the process of creating an image but not when the
            # image is actually booting with this initrd
            exclude_from_archive = [
                '/' + Defaults.get_shared_cache_location(),
                '/image', '/usr/lib/grub*'
            ]
            # the following is a list of directories to exclude which
            # are not needed inside of the initrd
            exclude_from_archive += [
                '/usr/share/doc', '/usr/share/man', '/home', '/media', '/srv'
            ]
            cpio.create(
                source_dir=temp_boot_root_directory,
                exclude=exclude_from_archive
            )
            log.info(
                '--> xz compressing archive'
            )
            compress = Compress(
                os.sep.join([self.target_dir, kiwi_initrd_basename])
            )
            compress.xz(
                ['--check=crc32', '--lzma2=dict=1MiB', '--threads=0']
            )
            self.initrd_filename = compress.compressed_filename

    def cleanup(self) -> None:
        for directory in self.temp_directories:
            if directory and os.path.exists(directory):
                Path.wipe(directory)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from string import Template
from textwrap import dedent


class VmwareSettingsTemplate:
    """
    VMware machine settings template
    """
    def __init__(self):
        self.cr = '\n'

        self.header = dedent('''
            #!/usr/bin/env vmware
            # kiwi generated VMware settings file
            config.version = "8"
            tools.syncTime = "true"
            uuid.action = "create"
            virtualHW.version = "${virtual_hardware_version}"
            displayName = "${display_name}"
            guestOS = "${guest_os}"
        ''').strip() + self.cr

        self.ide_disk = dedent('''
            ide${disk_id}:0.present = "true"
            ide${disk_id}:0.fileName= "${vmdk_file}"
            ide${disk_id}:0.redo = ""
        ''').strip() + self.cr

        self.scsi_disk = dedent('''
            scsi${disk_id}.present = "true"
            scsi${disk_id}.sharedBus = "none"
            scsi${disk_id}.virtualDev = "${scsi_controller_name}"
            scsi${disk_id}:0.present = "true"
            scsi${disk_id}:0.fileName = "${vmdk_file}"
            scsi${disk_id}:0.deviceType = "scsi-hardDisk"
        ''').strip() + self.cr

        self.network = dedent('''
            ethernet{nic_id}.present = "true"
            ethernet{nic_id}.allow64bitVmxnet = "true"
        ''').strip() + self.cr

        self.network_mac_static = dedent('''
            ethernet{nic_id}.addressType = "static"
            ethernet{nic_id}.address = "{mac_address}"
        ''').strip() + self.cr

        self.network_mac_generated = dedent('''
            ethernet{nic_id}.addressType = "generated"
        ''').strip() + self.cr

        self.network_driver = dedent('''
            ethernet{nic_id}.virtualDev = "{network_driver}"
        ''').strip() + self.cr

        self.network_connection_type = dedent('''
            ethernet{nic_id}.connectionType = "{mode}"
        ''').strip() + self.cr

        self.memory = dedent('''
            memsize = "${memory_size}"
        ''').strip() + self.cr

        self.number_of_cpus = dedent('''
            numvcpus = "${number_of_cpus}"
        ''').strip() + self.cr

        self.ide_iso = dedent('''
            ide${iso_id}:0.present = "true"
            ide${iso_id}:0.deviceType = "cdrom-raw"
            ide${iso_id}:0.autodetect = "true"
            ide${iso_id}:0.startConnected = "true"
        ''').strip() + self.cr

        self.scsi_iso = dedent('''
            scsi${iso_id}:0.present = "true"
            scsi${iso_id}:0.deviceType = "cdrom-raw"
            scsi${iso_id}:0.autodetect = "true"
            scsi${iso_id}:0.startConnected = "true"
        ''').strip() + self.cr

        self.usb = dedent('''
            usb.present = "true"
        ''').strip() + self.cr

        self.defaults = dedent('''
            priority.grabbed = "normal"
            priority.ungrabbed = "normal"
            powerType.powerOff = "soft"
            powerType.powerOn  = "soft"
            powerType.suspend  = "soft"
            powerType.reset    = "soft"
        ''').strip() + self.cr

    def get_template(
        self, memory_setup=False, cpu_setup=False, network_setup=False,
        iso_setup=False, disk_controller='ide', iso_controller='ide'
    ):
        """
        VMware machine configuration template

        :param bool memory_setup: with main memory setup true|false
        :param bool cpu_setup: with number of CPU's setup true|false
        :param bool network_setup: with network emulation true|false
        :param bool iso_setup: with CD/DVD drive emulation true|false
        :param string disk_controller: add disk controller setup to template
        :param string iso_controller: add CD/DVD controller setup to template
        :param string network_mac: add static MAC address setup to template
        :param string network_driver: add network driver setup to template
        :param string network_connection_type: add connection type to template

        :rtype: Template
        """
        template_data = self.header
        template_data += self.defaults

        if memory_setup:
            template_data += self.memory

        if cpu_setup:
            template_data += self.number_of_cpus

        if disk_controller == 'ide':
            template_data += self.ide_disk
        else:
            template_data += self.scsi_disk

        if network_setup:
            for nic_id, nic_setup in list(network_setup.items()):
                template_data += self.network.format(nic_id=nic_id)
                if nic_setup['mac'] == 'generated':
                    template_data += self.network_mac_generated.format(
                        nic_id=nic_id
                    )
                else:
                    template_data += self.network_mac_static.format(
                        nic_id=nic_id, mac_address=nic_setup['mac']
                    )
                if nic_setup['driver']:
                    template_data += self.network_driver.format(
                        nic_id=nic_id, network_driver=nic_setup['driver']
                    )
                if nic_setup['connection_type']:
                    template_data += self.network_connection_type.format(
                        nic_id=nic_id, mode=nic_setup['connection_type']
                    )

        if iso_setup:
            if iso_controller == 'ide':
                template_data += self.ide_iso
            else:
                template_data += self.scsi_iso

        template_data += self.usb

        return Template(template_data)



from mock import (
    call, patch
)
from pytest import raises

from kiwi.defaults import Defaults
from kiwi.iso_tools.iso import Iso

from kiwi.exceptions import KiwiIsoLoaderError


class TestIso:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        self.iso = Iso('source-dir')

    @patch('os.path.exists')
    def test_setup_isolinux_boot_path_raises(self, mock_exists):
        mock_exists.return_value = False
        with raises(KiwiIsoLoaderError):
            self.iso.setup_isolinux_boot_path()

    @patch('os.path.exists')
    @patch('kiwi.iso_tools.iso.Command.run')
    def test_setup_isolinux_boot_path(self, mock_command, mock_exists):
        mock_exists.return_value = True
        self.iso.setup_isolinux_boot_path()
        mock_command.assert_called_once_with(
            [
                'isolinux-config', '--base', 'boot/x86_64/loader',
                'source-dir/boot/x86_64/loader/isolinux.bin'
            ]
        )

    @patch('kiwi.iso_tools.iso.Command.run')
    @patch('os.path.exists')
    def test_setup_isolinux_boot_path_failed_isolinux_config(
        self, mock_exists, mock_command
    ):
        mock_exists.return_value = True
        command_raises = [False, True]

        def side_effect(arg):
            if command_raises.pop():
                raise Exception

        mock_command.side_effect = side_effect
        self.iso.setup_isolinux_boot_path()
        assert mock_command.call_args_list[1] == call(
            [
                'cp', '-a', '-l',
                'source-dir/boot/x86_64/loader/', 'source-dir/isolinux/'
            ]
        )

    @patch('kiwi.iso_tools.iso.Command.run')
    def test_set_media_tag(self, mock_command):
        Iso.set_media_tag('foo')
        mock_command.assert_called_once_with(
            ['tagmedia', '--md5', '--check', '--pad', '150', 'foo']
        )



# Copyright (c) 2020 SUSE LLC.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from typing import Any
import importlib

# project
from kiwi.utils.temporary import Temporary
from kiwi.markup.base import MarkupBase

from kiwi.exceptions import (
    KiwiDescriptionInvalid,
    KiwiAnyMarkupPluginError
)


class MarkupAny(MarkupBase):
    """
    **Implements markup handling for XML, YAML, JSON and INI**
    """
    def post_init(self) -> None:
        """
        Convert given description file into XML

        The anymarkup module supports auto detection of the given
        input format and can convert YAML, JSON and INI to XML
        """
        try:
            self.anymarkup: Any = importlib.import_module('anymarkup')
        except Exception as issue:
            raise KiwiAnyMarkupPluginError(issue)
        try:
            self.description_markup_processed = Temporary().new_file()
            markup = self.anymarkup.parse_file(
                self.description, force_types=None
            )
            self.anymarkup.serialize_file(
                markup, self.description_markup_processed.name, format='xml'
            )
        except Exception as issue:
            raise KiwiDescriptionInvalid(issue)

    def get_xml_description(self) -> str:
        """
        Return XML description file name

        :return: file path name

        :rtype: str
        """
        return self.apply_xslt_stylesheets(
            self.description_markup_processed.name
        )

    def get_yaml_description(self) -> str:
        """
        Return YAML description file name

        :return: file path name

        :rtype: str
        """
        xml_description_xslt_transformed = self.apply_xslt_stylesheets(
            self.description_markup_processed.name
        )
        markup = self.anymarkup.parse_file(
            xml_description_xslt_transformed, force_types=None
        )
        # The translation to yaml runs from a translation to json first
        # This is done because a direct translation from xml to yaml
        # causes the ordered mapping to be included. See
        # http://yaml.org/type/omap.html. In the context of kiwi this
        # ordering information is however not needed. The order of
        # sections and attributes doesn't play a role. In json no
        # such ordering concept exists and this is way to convert to
        # yaml without the !!omap definitions.
        self.anymarkup.serialize_file(
            markup, xml_description_xslt_transformed, format='json'
        )
        markup = self.anymarkup.parse_file(
            xml_description_xslt_transformed, force_types=None
        )
        self.anymarkup.serialize_file(
            markup, xml_description_xslt_transformed, format='yaml'
        )
        return xml_description_xslt_transformed



import pytest


def test_does_nothing_when_product_correct(shared_container):
    previous_contents = shared_container.file("/etc/products.d").listdir()
    shared_container.run_expect([0], ". /bin/functions.sh && suseSetupProduct")
    assert (
        shared_container.file("/etc/products.d").listdir() == previous_contents
    )


@pytest.mark.parametrize(
    "shared_container,product_name",
    (
        ("Tumbleweed", "openSUSE.prod"),
        ("Leap-15.2", "openSUSE.prod"),
        ("Leap-15.3", "Leap.prod"),
        ("SLE-15-SP3", "SLES.prod"),
        ("SLE-15-SP2", "SLES.prod"),
        ("SLE-15-SP1", "SLES.prod"),
        ("SLE-12-SP5", "SLES.prod"),
    ),
    indirect=["shared_container"],
)
def test_sets_baseproduct_from_etc_os_relesae(shared_container, product_name):
    assert not shared_container.file("/etc/SuSE-brand").exists

    shared_container.run_expect([0], "rm /etc/products.d/baseproduct")
    shared_container.run_expect([0], ". /bin/functions.sh && suseSetupProduct")

    assert shared_container.file("/etc/products.d/baseproduct").exists
    assert shared_container.file("/etc/products.d/baseproduct").is_symlink
    assert (
        shared_container.file(
            "/etc/products.d/baseproduct"
        ).linked_to == "/etc/products.d/" + product_name
    )


def test_sets_baseproduct_with_weird_os_release(container_per_test):
    assert not container_per_test.file("/etc/SuSE-brand").exists
    container_per_test.run_expect([0], "rm /etc/products.d/baseproduct")
    container_per_test.run_expect(
        [0],
        """cat <<EOF > /etc/os-release
NAME=openSUSE Tumbleweed
ID="opensuse-tumbleweed"
ID_LIKE="opensuse suse"
EOF
""",
    )

    container_per_test.run_expect(
        [0], ". /bin/functions.sh && suseSetupProduct"
    )
    assert container_per_test.file("/etc/products.d/baseproduct").exists
    assert container_per_test.file("/etc/products.d/baseproduct").is_symlink
    assert (
        container_per_test.file(
            "/etc/products.d/baseproduct"
        ).linked_to == "/etc/products.d/openSUSE.prod"
    )


def test_sets_baseproduct_from_prod_files(container_per_test):
    container_per_test.run_expect(
        [0],
        """rm /etc/products.d/baseproduct
rm /etc/products.d/openSUSE.prod
touch /etc/products.d/10.prod
touch /etc/products.d/20.prod
""",
    )

    container_per_test.run_expect(
        [0], ". /bin/functions.sh && suseSetupProduct"
    )
    assert container_per_test.file("/etc/products.d/baseproduct").exists
    assert container_per_test.file("/etc/products.d/baseproduct").is_symlink
    assert (
        container_per_test.file(
            "/etc/products.d/baseproduct"
        ).linked_to == "/etc/products.d/20.prod"
    )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import logging
from typing import Optional

# project
from kiwi.utils.temporary import Temporary
from kiwi.command import Command
from kiwi.defaults import Defaults
from kiwi.storage.device_provider import DeviceProvider
from kiwi.storage.mapped_device import MappedDevice

from kiwi.exceptions import (
    KiwiLuksSetupError
)

log = logging.getLogger('kiwi')


class LuksDevice(DeviceProvider):
    """
    **Implements luks setup on a storage device**

    :param object storage_provider: Instance of class based on DeviceProvider
    """
    def __init__(self, storage_provider: DeviceProvider) -> None:
        # bind the underlaying block device providing class instance
        # to this object (e.g loop) if present. This is done to guarantee
        # the correct destructor order when the device should be released.
        self.storage_provider = storage_provider

        self.luks_device: Optional[str] = None
        self.luks_keyfile: Optional[str] = None
        self.luks_name = 'luksRoot'

        self.option_map = {
            'sle12': [
                '--cipher', 'aes-xts-plain64',
                '--key-size', '256',
                '--hash', 'sha1'
            ]
        }

    def get_device(self) -> Optional[MappedDevice]:
        """
        Instance of MappedDevice providing the luks device

        :return: mapped luks device

        :rtype: MappedDevice
        """
        if self.luks_device:
            return MappedDevice(
                device=self.luks_device, device_provider=self
            )
        return None

    def create_crypto_luks(
        self, passphrase: str, os: str = None,
        options: list = None, keyfile: str = ''
    ) -> None:
        """
        Create luks device. Please note the passphrase is readable
        at creation time of this image. Make sure your host system
        is secure while this process runs

        :param string passphrase: credentials
        :param string os:
            distribution name to match distribution specific
            options for cryptsetup
        :param list options: further cryptsetup options
        :param string keyfile: file path name
            file path name which contains an alternative key
            to unlock the luks device
        """
        if not options:
            options = []
        if os:
            if os in self.option_map:
                options += self.option_map[os]
            else:
                raise KiwiLuksSetupError(
                    'no custom option configuration found for OS %s' % os
                )
        extra_options = []
        storage_device = self.storage_provider.get_device()
        log.info('Creating crypto LUKS on %s', storage_device)

        if not passphrase:
            log.warning('Using an empty passphrase for the key setup')

        log.info('--> Randomizing...')
        storage_size_mbytes = self.storage_provider.get_byte_size(
            storage_device
        ) / 1048576
        Command.run(
            [
                'dd', 'if=/dev/urandom', 'bs=1M',
                'count=%d' % storage_size_mbytes,
                'of=%s' % storage_device
            ]
        )
        log.info('--> Creating LUKS map')

        if passphrase:
            passphrase_file_tmp = Temporary().new_file()
            with open(passphrase_file_tmp.name, 'w') as credentials:
                credentials.write(passphrase)
            passphrase_file = passphrase_file_tmp.name
        else:
            passphrase_file_zero = '/dev/zero'
            extra_options = [
                '--keyfile-size', '32'
            ]
            passphrase_file = passphrase_file_zero

        Command.run(
            [
                'cryptsetup', '-q', '--key-file', passphrase_file
            ] + options + extra_options + [
                'luksFormat', storage_device
            ]
        )
        if keyfile:
            self.luks_keyfile = keyfile
            LuksDevice.create_random_keyfile(keyfile)
            Command.run(
                [
                    'cryptsetup', '--key-file', passphrase_file
                ] + extra_options + [
                    'luksAddKey', storage_device, keyfile
                ]
            )
        Command.run(
            [
                'cryptsetup', '--key-file', passphrase_file
            ] + extra_options + [
                'luksOpen', storage_device, self.luks_name
            ]
        )
        self.luks_device = '/dev/mapper/' + self.luks_name

    def create_crypttab(self, filename: str) -> None:
        """
        Create crypttab, setting the UUID of the storage device

        :param string filename: file path name
        """
        storage_device = self.storage_provider.get_device()
        with open(filename, 'w') as crypttab:
            luks_uuid = self.storage_provider.get_uuid(storage_device)
            if self.luks_keyfile:
                crypttab.write(
                    'luks UUID={0} /{1}{2}'.format(
                        luks_uuid, os.path.basename(self.luks_keyfile),
                        os.linesep
                    )
                )
            else:
                crypttab.write(
                    'luks UUID={0}{1}'.format(
                        luks_uuid, os.linesep
                    )
                )

    def is_loop(self) -> bool:
        """
        Check if storage provider is loop based

        Return loop status from base storage provider

        :return: True or False

        :rtype: bool
        """
        return self.storage_provider.is_loop()

    @staticmethod
    def create_random_keyfile(filename: str) -> None:
        """
        Create keyfile with random data

        :param string filename: file path name
        """
        with open(filename, 'wb') as keyfile:
            keyfile.write(os.urandom(Defaults.get_luks_key_length()))
        os.chmod(filename, 0o600)

    def __del__(self):
        if self.luks_device:
            log.info('Cleaning up %s instance', type(self).__name__)
            try:
                Command.run(
                    ['cryptsetup', 'luksClose', self.luks_name]
                )
            except Exception:
                log.warning(
                    'Shutdown of luks map %s failed, %s still busy',
                    self.luks_name, self.luks_device
                )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from typing import (
    Dict, List
)
import logging

# project
from kiwi.utils.temporary import Temporary
from kiwi.filesystem.base import FileSystemBase
from kiwi.filesystem.ext4 import FileSystemExt4
from kiwi.command import Command
from kiwi.system.size import SystemSize
from kiwi.storage.loop_device import LoopDevice

log = logging.getLogger('kiwi')


class FileSystemClicFs(FileSystemBase):
    """
    **Implements creation of clicfs filesystem**
    """
    def post_init(self, custom_args: Dict = None):
        """
        Post initialization method

        Initialize temporary container_dir directory to store
        clicfs embeded filesystem

        :param dict custom_args: unused
        """
        pass

    def create_on_file(
        self, filename: str, label: str = None, exclude: List[str] = None
    ):
        """
        Create clicfs filesystem from data tree

        There is no label which could be set for clicfs
        thus this parameter is not used

        There is no option to exclude data from clicfs
        thus this parameter is not used

        :param string filename: result file path name
        :param string label: unused
        :param list exclude: unused
        """
        container_dir = Temporary(prefix='kiwi_clicfs.').new_dir()
        clicfs_container_filesystem = container_dir.name + '/fsdata.ext4'
        loop_provider = LoopDevice(
            clicfs_container_filesystem,
            self._get_container_filesystem_size_mbytes()
        )
        loop_provider.create()
        filesystem = FileSystemExt4(
            loop_provider, self.root_dir
        )
        filesystem.create_on_device()
        filesystem.sync_data()
        Command.run(
            ['resize2fs', '-f', loop_provider.get_device(), '-M']
        )

        # force cleanup and umount of container filesystem
        # before mkclicfs is called
        del filesystem

        Command.run(
            ['mkclicfs', clicfs_container_filesystem, filename]
        )

    def _get_container_filesystem_size_mbytes(self):
        size = SystemSize(self.root_dir)
        root_dir_mbytes = size.accumulate_mbyte_file_sizes()
        return size.customize(root_dir_mbytes, 'ext4')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from os import path
from setuptools import setup
from setuptools.command import sdist as setuptools_sdist

import distutils
import subprocess

from kiwi.version import __version__

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as readme:
    long_description = readme.read()


class sdist(setuptools_sdist.sdist):
    """
    Custom sdist command
    Host requirements: git
    """
    def run(self):
        """
        Run first the git commit format update $Format:%H$
        and after that the usual Python sdist
        """
        # git attributes
        command = ['make', 'git_attributes']
        self.announce(
            'Running make git_attributes target: %s' % str(command),
            level=distutils.log.INFO
        )
        self.announce(
            subprocess.check_output(command).decode(),
            level=distutils.log.INFO
        )

        # standard sdist process
        setuptools_sdist.sdist.run(self)

        # cleanup attributes
        command = ['make', 'clean_git_attributes']
        self.announce(
            subprocess.check_output(command).decode(),
            level=distutils.log.INFO
        )


config = {
    'name': 'kiwi',
    'long_description': long_description,
    'long_description_content_type': 'text/x-rst',
    'python_requires': '>=3.6',
    'description': 'KIWI - Appliance Builder (next generation)',
    'author': 'Marcus Schaefer',
    'url': 'https://osinside.github.io/kiwi',
    'download_url':
        'https://download.opensuse.org/repositories/'
        'Virtualization:/Appliances:/Builder',
    'author_email': 'ms@suse.com',
    'version': __version__,
    'license' : 'GPLv3+',
    'install_requires': [
        'docopt>=0.6.2',
        'lxml',
        'pyxattr',
        'requests',
        'PyYAML',
        'simplejson'
    ],
    'packages': ['kiwi'],
    'cmdclass': {
        'sdist': sdist
    },
    'entry_points': {
        'kiwi.tasks': [
            'image_info=kiwi.tasks.image_info',
            'image_resize=kiwi.tasks.image_resize',
            'result_bundle=kiwi.tasks.result_bundle',
            'result_list=kiwi.tasks.result_list',
            'system_build=kiwi.tasks.system_build',
            'system_create=kiwi.tasks.system_create',
            'system_prepare=kiwi.tasks.system_prepare',
            'system_update=kiwi.tasks.system_update'
        ],
        'console_scripts': [
            'kiwi-ng=kiwi.kiwi:main',
            'kiwicompat=kiwi.kiwi_compat:main'
        ]
    },
    'include_package_data': True,
    'zip_safe': False,
    'classifiers': [
       # classifier: http://pypi.python.org/pypi?%3Aaction=list_classifiers
       'Development Status :: 5 - Production/Stable',
       'Intended Audience :: Developers',
       'License :: OSI Approved :: '
       'GNU General Public License v3 or later (GPLv3+)',
       'Operating System :: POSIX :: Linux',
       'Programming Language :: Python :: 3.6',
       'Programming Language :: Python :: 3.7',
       'Topic :: System :: Operating System',
    ]
}

setup(**config)



import logging
from pytest import fixture
from mock import patch
import mock

from kiwi.filesystem.isofs import FileSystemIsoFs


class TestFileSystemIsoFs:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    @patch('os.path.exists')
    def setup(self, mock_exists):
        mock_exists.return_value = True
        self.isofs = FileSystemIsoFs(mock.Mock(), 'root_dir')

    def test_post_init(self):
        self.isofs.post_init({'some_args': 'data'})
        assert self.isofs.custom_args['meta_data'] == {}
        assert self.isofs.custom_args['mount_options'] == []
        assert self.isofs.custom_args['some_args'] == 'data'

    @patch('kiwi.filesystem.isofs.IsoTools')
    @patch('kiwi.filesystem.isofs.Iso')
    def test_create_on_file(self, mock_iso, mock_IsoTools):
        iso_tool = mock.Mock()
        iso_tool.has_iso_hybrid_capability = mock.Mock(
            return_value=True
        )
        iso_tool.get_tool_name = mock.Mock(
            return_value='/usr/bin/xorriso'
        )
        mock_IsoTools.new.return_value = iso_tool
        iso = mock.Mock()
        iso.header_end_name = 'header_end'
        mock_iso.return_value = iso
        self.isofs.create_on_file('myimage', None)

        iso.setup_isolinux_boot_path.assert_called_once_with()

        iso_tool.init_iso_creation_parameters.assert_called_once_with({})

        iso_tool.add_efi_loader_parameters.assert_called_once_with()

        iso_tool.create_iso.assert_called_once_with('myimage')

    @patch('kiwi.filesystem.isofs.IsoTools')
    @patch('kiwi.filesystem.isofs.Iso')
    def test_create_on_file_EFI_enabled(self, mock_iso, mock_IsoTools):
        iso_tool = mock.Mock()
        iso_tool.has_iso_hybrid_capability = mock.Mock(
            return_value=True
        )
        iso_tool.get_tool_name = mock.Mock(
            return_value='/usr/bin/xorriso'
        )
        mock_IsoTools.new.return_value = iso_tool
        iso = mock.Mock()
        iso.header_end_name = 'header_end'
        mock_iso.return_value = iso
        self.isofs.custom_args['meta_data']['efi_mode'] = 'uefi'
        with self._caplog.at_level(logging.WARNING):
            self.isofs.create_on_file('myimage')
            iso_tool.create_iso.assert_called_once_with('myimage')



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os

# project
from kiwi.defaults import Defaults
from kiwi.command import Command
from kiwi.exceptions import KiwiIsoLoaderError


class Iso:
    """
    **Implements helper methods around the creation of ISO filesystems**
    """
    def __init__(self, source_dir: str) -> None:
        """
        Create an instance of Iso helpers

        :param str source_dir:
            Directory path for iso source dir, root of ISO
        """
        self.source_dir = source_dir
        self.boot_path = Defaults.get_iso_boot_path()

    @staticmethod
    def set_media_tag(isofile: str) -> None:
        """
        Include checksum tag in the ISO so it can be verified with
        the mediacheck program.

        :param str isofile: path to the ISO file
        """
        Command.run(
            [
                'tagmedia',
                '--md5',
                '--check',
                '--pad', '150',
                isofile
            ]
        )

    def setup_isolinux_boot_path(self) -> None:
        """
        Write the base boot path into the isolinux loader binary

        :raises KiwiIsoLoaderError: if loader/isolinux.bin is not found
        """
        loader_base_directory = self.boot_path + '/loader'
        loader_file = '/'.join(
            [self.source_dir, self.boot_path, 'loader/isolinux.bin']
        )
        if not os.path.exists(loader_file):
            raise KiwiIsoLoaderError(
                'No isolinux loader {} found'.format(loader_file)
            )
        try:
            Command.run(
                [
                    'isolinux-config', '--base', loader_base_directory,
                    loader_file
                ]
            )
        except Exception:
            # Setup of the base directory failed. This happens if
            # isolinux-config was not able to identify the isolinux
            # signature. As a workaround a compat directory /isolinux
            # is created which hardlinks all loader files
            loader_source_directory = os.sep.join(
                [self.source_dir, loader_base_directory]
            )
            loader_compat_target_directory = os.sep.join(
                [self.source_dir, 'isolinux']
            )
            Command.run(
                [
                    'cp', '-a', '-l',
                    loader_source_directory + os.sep,
                    loader_compat_target_directory + os.sep
                ]
            )
